@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009, 2011, 2012  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2009, 2011  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1998-2003  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: t_names.c,v 1.50.346.2 2011/03/12 04:59:15 tbox Exp $ */
+/* $Id: t_names.c,v 1.52 2011/03/12 04:59:46 tbox Exp $ */
 
 #include <config.h>
 
@@ -27,7 +27,6 @@
 #include <isc/string.h>
 
 #include <dns/compress.h>
-#include <dns/fixedname.h>
 #include <dns/name.h>
 #include <dns/result.h>
 
@@ -122,30 +121,20 @@ chkdata(unsigned char *buf, size_t buflen, char *exp_data,
 
 			if (('0' <= *q) && (*q <= '9'))
 				*p = *q - '0';
-			else if (('a' <= *q) && (*q <= 'f'))
+			else if (('a' <= *q) && (*q <= 'z'))
 				*p = *q - 'a' + 10;
-			else if (('A' <= *q) && (*q <= 'F'))
+			else if (('A' <= *q) && (*q <= 'Z'))
 				*p = *q - 'A' + 10;
-			else {
-				t_info("malformed comparison data\n");
-				free(data);
-				return (-1);
-			}
 			++q;
 
 			*p <<= 4;
 
 			if (('0' <= *q) && (*q <= '9'))
 				*p |= ((*q - '0') & 0x0f);
-			else if (('a' <= *q) && (*q <= 'f'))
+			else if (('a' <= *q) && (*q <= 'z'))
 				*p |= ((*q - 'a' + 10) & 0x0f);
-			else if (('A' <= *q) && (*q <= 'F'))
+			else if (('A' <= *q) && (*q <= 'Z'))
 				*p |= ((*q - 'A' + 10) & 0x0f);
-			else {
-				t_info("malformed comparison data\n");
-				free(data);
-				return (-1);
-			}
 			++p;
 			++q;
 			++cnt;
@@ -167,7 +156,7 @@ chkdata(unsigned char *buf, size_t buflen, char *exp_data,
 		else {
 			t_info("bad data at position %lu, "
 			       "got 0x%.2x, expected 0x%.2x\n",
-			       (unsigned long)cnt, *p, *v);
+			       (unsigned long)cnt, *p, *q);
 			result = cnt + 1;
 		}
 		(void)free(data);
@@ -185,14 +174,13 @@ chkdata(unsigned char *buf, size_t buflen, char *exp_data,
  * setup the buffer and return the data length.
  */
 static int
-getmsg(char *datafile_name, isc_buffer_t *pbuf)
+getmsg(char *datafile_name, unsigned char *buf, int buflen, isc_buffer_t *pbuf)
 {
 	int			c;
-	unsigned int		len;
-	unsigned int		cnt;
+	int			len;
+	int			cnt;
 	unsigned char		*p;
 	FILE			*fp;
-	unsigned int		buflen;
 
 	fp = fopen(datafile_name, "r");
 	if (fp == NULL) {
@@ -200,8 +188,7 @@ getmsg(char *datafile_name, isc_buffer_t *pbuf)
 		return (0);
 	}
 
-	p = isc_buffer_used(pbuf);
-	buflen = isc_buffer_availablelength(pbuf);
+	p = buf;
 	cnt = 0;
 	len = 0;
 	while ((c = getc(fp)) != EOF) {
@@ -216,9 +203,9 @@ getmsg(char *datafile_name, isc_buffer_t *pbuf)
 		}
 		if (('0' <= c) && (c <= '9'))
 			val = c - '0';
-		else if (('a' <= c) && (c <= 'f'))
+		else if (('a' <= c) && (c <= 'z'))
 			val = c - 'a' + 10;
-		else if (('A' <= c) && (c <= 'F'))
+		else if (('A' <= c) && (c <= 'Z'))
 			val = c - 'A'+ 10;
 		else {
 			(void)fclose(fp);
@@ -250,6 +237,7 @@ getmsg(char *datafile_name, isc_buffer_t *pbuf)
 	}
 
 	*p = '\0';
+	isc_buffer_init(pbuf, buf, cnt);
 	isc_buffer_add(pbuf, cnt);
 	return (cnt);
 }
@@ -332,6 +320,40 @@ hname_to_tname(char *src, char *target, size_t len) {
 
 #endif	/* NEED_HNAME_TO_TNAME */
 
+/*%
+ * initialize a dns_name_t from a text name, hiding all
+ * buffer and other object initialization from the caller
+ *
+ */
+
+static isc_result_t
+dname_from_tname(char *name, dns_name_t *dns_name) {
+	int		len;
+	isc_buffer_t	txtbuf;
+	isc_buffer_t	*binbuf;
+	unsigned char	*junk;
+	isc_result_t	result;
+
+	len = strlen(name);
+	isc_buffer_init(&txtbuf, name, len);
+	isc_buffer_add(&txtbuf, len);
+	junk = (unsigned char *)malloc(sizeof(unsigned char) * BUFLEN);
+	binbuf = (isc_buffer_t *)malloc(sizeof(isc_buffer_t));
+	if ((junk != NULL) && (binbuf != NULL)) {
+		isc_buffer_init(binbuf, junk, BUFLEN);
+		dns_name_init(dns_name, NULL);
+		dns_name_setbuffer(dns_name, binbuf);
+		result = dns_name_fromtext(dns_name,  &txtbuf, NULL, 0, NULL);
+	} else {
+		result = ISC_R_NOSPACE;
+		if (junk != NULL)
+			(void)free(junk);
+		if (binbuf != NULL)
+			(void)free(binbuf);
+	}
+	return (result);
+}
+
 static const char *a3 =	"dns_name_init initializes 'name' to the empty name";
 
 static void
@@ -592,10 +614,8 @@ test_dns_name_hash(char *test_name1, char *test_name2,
 	isc_boolean_t	match;
 	unsigned int	hash1;
 	unsigned int	hash2;
-	dns_fixedname_t	fixed1;
-	dns_fixedname_t	fixed2;
-	dns_name_t	*dns_name1;
-	dns_name_t	*dns_name2;
+	dns_name_t	dns_name1;
+	dns_name_t	dns_name2;
 	isc_result_t	result;
 
 	rval = T_UNRESOLVED;
@@ -603,17 +623,12 @@ test_dns_name_hash(char *test_name1, char *test_name2,
 
 	t_info("testing names %s and %s\n", test_name1, test_name2);
 
-	dns_fixedname_init(&fixed1);
-	dns_fixedname_init(&fixed2);
-	dns_name1 = dns_fixedname_name(&fixed1);
-	dns_name2 = dns_fixedname_name(&fixed2);
-	result = dns_name_fromstring2(dns_name1, test_name1, NULL, 0, NULL);
+	result = dname_from_tname(test_name1, &dns_name1);
 	if (result == ISC_R_SUCCESS) {
-		result = dns_name_fromstring2(dns_name2, test_name2, NULL,
-					      0, NULL);
+		result = dname_from_tname(test_name2, &dns_name2);
 		if (result == ISC_R_SUCCESS) {
-			hash1 = dns_name_hash(dns_name1, ISC_TRUE);
-			hash2 = dns_name_hash(dns_name2, ISC_TRUE);
+			hash1 = dns_name_hash(&dns_name1, ISC_TRUE);
+			hash2 = dns_name_hash(&dns_name2, ISC_TRUE);
 			match = ISC_FALSE;
 			if (hash1 == hash2)
 				match = ISC_TRUE;
@@ -621,8 +636,8 @@ test_dns_name_hash(char *test_name1, char *test_name2,
 				++failures;
 				t_info("hash mismatch when ISC_TRUE\n");
 			}
-			hash1 = dns_name_hash(dns_name1, ISC_FALSE);
-			hash2 = dns_name_hash(dns_name2, ISC_FALSE);
+			hash1 = dns_name_hash(&dns_name1, ISC_FALSE);
+			hash2 = dns_name_hash(&dns_name2, ISC_FALSE);
 			match = ISC_FALSE;
 			if (hash1 == hash2)
 				match = ISC_TRUE;
@@ -635,11 +650,11 @@ test_dns_name_hash(char *test_name1, char *test_name2,
 			else
 				rval = T_FAIL;
 		} else {
-			t_info("dns_name_fromstring2 %s failed, result = %s\n",
+			t_info("dns_fromtext %s failed, result = %s\n",
 				test_name2, dns_result_totext(result));
 		}
 	} else {
-		t_info("dns_name_fromstring2 %s failed, result = %s\n",
+		t_info("dns_fromtext %s failed, result = %s\n",
 				test_name1, dns_result_totext(result));
 	}
 	return (rval);
@@ -741,10 +756,8 @@ test_dns_name_fullcompare(char *name1, char *name2,
 	int		nfails;
 	int		order;
 	unsigned int	nlabels;
-	dns_fixedname_t fixed1;
-	dns_fixedname_t fixed2;
-	dns_name_t	*dns_name1;
-	dns_name_t	*dns_name2;
+	dns_name_t	dns_name1;
+	dns_name_t	dns_name2;
 	isc_result_t	dns_result;
 	dns_namereln_t	dns_reln;
 
@@ -755,16 +768,11 @@ test_dns_name_fullcompare(char *name1, char *name2,
 	t_info("testing names %s and %s for relation %s\n", name1, name2,
 	       dns_namereln_to_text(exp_dns_reln));
 
-	dns_fixedname_init(&fixed1);
-	dns_fixedname_init(&fixed2);
-	dns_name1 = dns_fixedname_name(&fixed1);
-	dns_name2 = dns_fixedname_name(&fixed2);
-	dns_result = dns_name_fromstring2(dns_name1, name1, NULL, 0, NULL);
+	dns_result = dname_from_tname(name1, &dns_name1);
 	if (dns_result == ISC_R_SUCCESS) {
-		dns_result = dns_name_fromstring2(dns_name2, name2, NULL,
-						  0, NULL);
+		dns_result = dname_from_tname(name2, &dns_name2);
 		if (dns_result == ISC_R_SUCCESS) {
-			dns_reln = dns_name_fullcompare(dns_name1, dns_name2,
+			dns_reln = dns_name_fullcompare(&dns_name1, &dns_name2,
 					&order, &nlabels);
 
 			if (dns_reln != exp_dns_reln) {
@@ -796,11 +804,11 @@ test_dns_name_fullcompare(char *name1, char *name2,
 			else
 				result = T_FAIL;
 		} else {
-			t_info("dns_name_fromstring2 failed, result == %s\n",
+			t_info("dname_from_tname failed, result == %s\n",
 			       dns_result_totext(result));
 		}
 	} else {
-		t_info("dns_name_fromstring2 failed, result == %s\n",
+		t_info("dname_from_tname failed, result == %s\n",
 		       dns_result_totext(result));
 	}
 
@@ -889,10 +897,8 @@ test_dns_name_compare(char *name1, char *name2, int exp_order) {
 	int		result;
 	int		order;
 	isc_result_t	dns_result;
-	dns_fixedname_t fixed1;
-	dns_fixedname_t fixed2;
-	dns_name_t	*dns_name1;
-	dns_name_t	*dns_name2;
+	dns_name_t	dns_name1;
+	dns_name_t	dns_name2;
 
 	result = T_UNRESOLVED;
 
@@ -900,16 +906,11 @@ test_dns_name_compare(char *name1, char *name2, int exp_order) {
 	       exp_order == 0 ? "==": (exp_order == -1 ? "<" : ">"),
 	       name2);
 
-	dns_fixedname_init(&fixed1);
-	dns_fixedname_init(&fixed2);
-	dns_name1 = dns_fixedname_name(&fixed1);
-	dns_name2 = dns_fixedname_name(&fixed2);
-	dns_result = dns_name_fromstring2(dns_name1, name1, NULL, 0, NULL);
+	dns_result = dname_from_tname(name1, &dns_name1);
 	if (dns_result == ISC_R_SUCCESS) {
-		dns_result = dns_name_fromstring2(dns_name2, name2, NULL,
-						  0, NULL);
+		dns_result = dname_from_tname(name2, &dns_name2);
 		if (dns_result == ISC_R_SUCCESS) {
-			order = dns_name_compare(dns_name1, dns_name2);
+			order = dns_name_compare(&dns_name1, &dns_name2);
 			/*
 			 * Normalize order.
 			 */
@@ -924,11 +925,11 @@ test_dns_name_compare(char *name1, char *name2, int exp_order) {
 			} else
 				result = T_PASS;
 		} else {
-			t_info("dns_name_fromstring2 failed, result == %s\n",
+			t_info("dname_from_tname failed, result == %s\n",
 					dns_result_totext(result));
 		}
 	} else {
-		t_info("dns_name_fromstring2 failed, result == %s\n",
+		t_info("dname_from_tname failed, result == %s\n",
 				dns_result_totext(result));
 	}
 
@@ -999,26 +1000,19 @@ test_dns_name_rdatacompare(char *name1, char *name2, int exp_order) {
 	int		result;
 	int		order;
 	isc_result_t	dns_result;
-	dns_fixedname_t fixed1;
-	dns_fixedname_t fixed2;
-	dns_name_t	*dns_name1;
-	dns_name_t	*dns_name2;
+	dns_name_t	dns_name1;
+	dns_name_t	dns_name2;
 
 	result = T_UNRESOLVED;
 
 	t_info("testing %s %s %s\n", name1,
 	       exp_order == 0 ? "==": (exp_order == -1 ? "<" : ">"), name2);
 
-	dns_fixedname_init(&fixed1);
-	dns_fixedname_init(&fixed2);
-	dns_name1 = dns_fixedname_name(&fixed1);
-	dns_name2 = dns_fixedname_name(&fixed2);
-	dns_result = dns_name_fromstring2(dns_name1, name1, NULL, 0, NULL);
+	dns_result = dname_from_tname(name1, &dns_name1);
 	if (dns_result == ISC_R_SUCCESS) {
-		dns_result = dns_name_fromstring2(dns_name2, name2, NULL,
-						  0, NULL);
+		dns_result = dname_from_tname(name2, &dns_name2);
 		if (dns_result == ISC_R_SUCCESS) {
-			order = dns_name_rdatacompare(dns_name1, dns_name2);
+			order = dns_name_rdatacompare(&dns_name1, &dns_name2);
 			/*
 			 * Normalize order.
 			 */
@@ -1033,11 +1027,11 @@ test_dns_name_rdatacompare(char *name1, char *name2, int exp_order) {
 			} else
 				result = T_PASS;
 		} else {
-			t_info("dns_name_fromstring2 failed, result == %s\n",
+			t_info("dname_from_tname failed, result == %s\n",
 			       dns_result_totext(result));
 		}
 	} else {
-		t_info("dns_name_fromstring2 failed, result == %s\n",
+		t_info("dname_from_tname failed, result == %s\n",
 		       dns_result_totext(result));
 	}
 
@@ -1109,26 +1103,19 @@ test_dns_name_issubdomain(char *name1, char *name2, isc_boolean_t exp_rval) {
 	int		result;
 	isc_boolean_t	rval;
 	isc_result_t	dns_result;
-	dns_fixedname_t fixed1;
-	dns_fixedname_t fixed2;
-	dns_name_t	*dns_name1;
-	dns_name_t	*dns_name2;
+	dns_name_t	dns_name1;
+	dns_name_t	dns_name2;
 
 	result = T_UNRESOLVED;
 
 	t_info("testing %s %s a subdomain of %s\n", name1,
 	       exp_rval == 0 ? "is not" : "is", name2);
 
-	dns_fixedname_init(&fixed1);
-	dns_fixedname_init(&fixed2);
-	dns_name1 = dns_fixedname_name(&fixed1);
-	dns_name2 = dns_fixedname_name(&fixed2);
-	dns_result = dns_name_fromstring2(dns_name1, name1, NULL, 0, NULL);
+	dns_result = dname_from_tname(name1, &dns_name1);
 	if (dns_result == ISC_R_SUCCESS) {
-		dns_result = dns_name_fromstring2(dns_name2, name2, NULL,
-						  0, NULL);
+		dns_result = dname_from_tname(name2, &dns_name2);
 		if (dns_result == ISC_R_SUCCESS) {
-			rval = dns_name_issubdomain(dns_name1, dns_name2);
+			rval = dns_name_issubdomain(&dns_name1, &dns_name2);
 
 			if (rval != exp_rval) {
 				t_info("expected return value of %s, got %s\n",
@@ -1138,11 +1125,11 @@ test_dns_name_issubdomain(char *name1, char *name2, isc_boolean_t exp_rval) {
 			} else
 				result = T_PASS;
 		} else {
-			t_info("dns_name_fromstring2 failed, result == %s\n",
+			t_info("dname_from_tname failed, result == %s\n",
 			       dns_result_totext(result));
 		}
 	} else {
-		t_info("dns_name_fromstring2 failed, result == %s\n",
+		t_info("dname_from_tname failed, result == %s\n",
 		       dns_result_totext(result));
 	}
 
@@ -1209,18 +1196,15 @@ test_dns_name_countlabels(char *test_name, unsigned int exp_nlabels) {
 	int		result;
 	unsigned int	nlabels;
 	isc_result_t	dns_result;
-	dns_fixedname_t	fixed;
-	dns_name_t	*dns_name;
+	dns_name_t	dns_name;
 
 	result = T_UNRESOLVED;
 
 	t_info("testing %s\n", test_name);
 
-	dns_fixedname_init(&fixed);
-	dns_name = dns_fixedname_name(&fixed);
-	dns_result = dns_name_fromstring2(dns_name, test_name, NULL, 0, NULL);
+	dns_result = dname_from_tname(test_name, &dns_name);
 	if (dns_result == ISC_R_SUCCESS) {
-		nlabels = dns_name_countlabels(dns_name);
+		nlabels = dns_name_countlabels(&dns_name);
 
 		if (nlabels != exp_nlabels) {
 			t_info("expected %d, got %d\n", exp_nlabels, nlabels);
@@ -1228,7 +1212,7 @@ test_dns_name_countlabels(char *test_name, unsigned int exp_nlabels) {
 		} else
 			result = T_PASS;
 	} else {
-		t_info("dns_name_fromstring2 failed, result == %s\n",
+		t_info("dname_from_tname failed, result == %s\n",
 		       dns_result_totext(dns_result));
 	}
 
@@ -1302,10 +1286,8 @@ test_dns_name_getlabel(char *test_name1, int label1_pos, char *test_name2,
 	unsigned int	cnt;
 	unsigned char	*p;
 	unsigned char	*q;
-	dns_fixedname_t fixed1;
-	dns_fixedname_t fixed2;
-	dns_name_t	*dns_name1;
-	dns_name_t	*dns_name2;
+	dns_name_t	dns_name1;
+	dns_name_t	dns_name2;
 	dns_label_t	dns_label1;
 	dns_label_t	dns_label2;
 	isc_result_t	dns_result;
@@ -1315,18 +1297,12 @@ test_dns_name_getlabel(char *test_name1, int label1_pos, char *test_name2,
 
 	t_info("testing with %s and %s\n", test_name1, test_name2);
 
-	dns_fixedname_init(&fixed1);
-	dns_fixedname_init(&fixed2);
-	dns_name1 = dns_fixedname_name(&fixed1);
-	dns_name2 = dns_fixedname_name(&fixed2);
-	dns_result = dns_name_fromstring2(dns_name1, test_name1, NULL,
-					  0, NULL);
+	dns_result = dname_from_tname(test_name1, &dns_name1);
 	if (dns_result == ISC_R_SUCCESS) {
-		dns_result = dns_name_fromstring2(dns_name2, test_name2, NULL,
-						  0, NULL);
+		dns_result = dname_from_tname(test_name2, &dns_name2);
 		if (dns_result == ISC_R_SUCCESS) {
-			dns_name_getlabel(dns_name1, label1_pos, &dns_label1);
-			dns_name_getlabel(dns_name2, label2_pos, &dns_label2);
+			dns_name_getlabel(&dns_name1, label1_pos, &dns_label1);
+			dns_name_getlabel(&dns_name2, label2_pos, &dns_label2);
 			if (dns_label1.length != dns_label2.length) {
 				t_info("label lengths differ\n");
 				++nfails;
@@ -1345,11 +1321,11 @@ test_dns_name_getlabel(char *test_name1, int label1_pos, char *test_name2,
 			else
 				result = T_FAIL;
 		} else {
-			t_info("dns_name_fromstring2 failed, result == %s",
+			t_info("dname_from_tname failed, result == %s",
 			       dns_result_totext(result));
 		}
 	} else {
-		t_info("dns_name_fromstring2 failed, result == %s",
+		t_info("dname_from_tname failed, result == %s",
 		       dns_result_totext(result));
 	}
 	return (result);
@@ -1423,10 +1399,8 @@ test_dns_name_getlabelsequence(char *test_name1, int label1_start,
 	unsigned int	cnt;
 	unsigned char	*p;
 	unsigned char	*q;
-	dns_fixedname_t fixed1;
-	dns_fixedname_t fixed2;
-	dns_name_t	*dns_name1;
-	dns_name_t	*dns_name2;
+	dns_name_t	dns_name1;
+	dns_name_t	dns_name2;
 	dns_name_t	dns_targetname1;
 	dns_name_t	dns_targetname2;
 	isc_result_t	dns_result;
@@ -1437,22 +1411,16 @@ test_dns_name_getlabelsequence(char *test_name1, int label1_start,
 
 	nfails = 0;
 	result = T_UNRESOLVED;
-	dns_fixedname_init(&fixed1);
-	dns_fixedname_init(&fixed2);
-	dns_name1 = dns_fixedname_name(&fixed1);
-	dns_name2 = dns_fixedname_name(&fixed2);
-	dns_result = dns_name_fromstring2(dns_name1, test_name1, NULL,
-					  0, NULL);
+	dns_result = dname_from_tname(test_name1, &dns_name1);
 	if (dns_result == ISC_R_SUCCESS) {
-		dns_result = dns_name_fromstring2(dns_name2, test_name2, NULL,
-						  0, NULL);
+		dns_result = dname_from_tname(test_name2, &dns_name2);
 		if (dns_result == ISC_R_SUCCESS) {
 			t_info("testing %s %s\n", test_name1, test_name2);
 			dns_name_init(&dns_targetname1, NULL);
 			dns_name_init(&dns_targetname2, NULL);
-			dns_name_getlabelsequence(dns_name1, label1_start,
+			dns_name_getlabelsequence(&dns_name1, label1_start,
 						  range, &dns_targetname1);
-			dns_name_getlabelsequence(dns_name2, label2_start,
+			dns_name_getlabelsequence(&dns_name2, label2_start,
 						  range, &dns_targetname2);
 
 			/*
@@ -1483,11 +1451,11 @@ test_dns_name_getlabelsequence(char *test_name1, int label1_start,
 			else
 				result = T_FAIL;
 		} else {
-			t_info("dns_name_fromstring2 failed, result == %s",
+			t_info("dname_from_tname failed, result == %s",
 			       dns_result_totext(dns_result));
 		}
 	} else {
-		t_info("dns_name_fromstring2 failed, result == %s",
+		t_info("dname_from_tname failed, result == %s",
 		       dns_result_totext(dns_result));
 	}
 	return (result);
@@ -1554,8 +1522,7 @@ test_dns_name_fromregion(char *test_name) {
 	int		order;
 	unsigned int	nlabels;
 	isc_result_t	dns_result;
-	dns_fixedname_t fixed1;
-	dns_name_t	*dns_name1;
+	dns_name_t	dns_name1;
 	dns_name_t	dns_name2;
 	dns_namereln_t	dns_namereln;
 	isc_region_t	region;
@@ -1564,23 +1531,21 @@ test_dns_name_fromregion(char *test_name) {
 
 	t_info("testing %s\n", test_name);
 
-	dns_fixedname_init(&fixed1);
-	dns_name1 = dns_fixedname_name(&fixed1);
-	dns_result = dns_name_fromstring2(dns_name1, test_name, NULL, 0, NULL);
+	dns_result = dname_from_tname(test_name, &dns_name1);
 	if (dns_result == ISC_R_SUCCESS) {
 
-		dns_name_toregion(dns_name1, &region);
+		dns_name_toregion(&dns_name1, &region);
 
 		dns_name_init(&dns_name2, NULL);
 		dns_name_fromregion(&dns_name2, &region);
-		dns_namereln = dns_name_fullcompare(dns_name1, &dns_name2,
+		dns_namereln = dns_name_fullcompare(&dns_name1, &dns_name2,
 						    &order, &nlabels);
 		if (dns_namereln == dns_namereln_equal)
 			result = T_PASS;
 		else
 			result = T_FAIL;
 	} else {
-		t_info("dns_name_fromstring2 failed, result == %s\n",
+		t_info("dname_from_tname failed, result == %s\n",
 		       dns_result_totext(result));
 	}
 	return (result);
@@ -2007,18 +1972,14 @@ test_dns_name_fromwire(char *datafile_name, int testname_offset, int downcase,
 	char			buf2[BUFLEN];
 	isc_buffer_t		iscbuf1;
 	isc_buffer_t		iscbuf2;
-	dns_fixedname_t		fixed2;
 	dns_name_t		dns_name1;
-	dns_name_t		*dns_name2;
+	dns_name_t		dns_name2;
 	isc_result_t		dns_result;
 	dns_namereln_t		dns_namereln;
 	dns_decompress_t	dctx;
 
 	t_info("testing using %s\n", datafile_name);
-	isc_buffer_init(&iscbuf1, buf1, sizeof(buf1));
-	len = getmsg(datafile_name, &iscbuf1);
-	if (len == 0)
-		return (T_FAIL);
+	len = getmsg(datafile_name, buf1, BIGBUFLEN, &iscbuf1);
 
 	isc_buffer_setactive(&iscbuf1, len);
 	iscbuf1.current = testname_offset;
@@ -2033,13 +1994,10 @@ test_dns_name_fromwire(char *datafile_name, int testname_offset, int downcase,
 
 	if ((dns_result == exp_result) && (exp_result == ISC_R_SUCCESS)) {
 
-		dns_fixedname_init(&fixed2);
-		dns_name2 = dns_fixedname_name(&fixed2);
-		dns_result = dns_name_fromstring2(dns_name2, exp_name, NULL,
-						  0, NULL);
+		dns_result = dname_from_tname(exp_name, &dns_name2);
 		if (dns_result == ISC_R_SUCCESS) {
 			dns_namereln = dns_name_fullcompare(&dns_name1,
-							    dns_name2,
+							    &dns_name2,
 							    &order, &nlabels);
 			if (dns_namereln != dns_namereln_equal) {
 				t_info("dns_name_fullcompare  returned %s\n",
@@ -2194,8 +2152,7 @@ static const char *a52 =
 
 static int
 test_dns_name_towire(char *testname, unsigned int dc_method, char *exp_data,
-		     size_t exp_data_len, isc_result_t exp_result,
-		     size_t buflen)
+		     int exp_data_len, isc_result_t exp_result, size_t buflen)
 {
 	int			result;
 	int			val;
@@ -2264,7 +2221,6 @@ t_dns_name_towire_x(const char *testfile, size_t buflen) {
 	int		result;
 	unsigned int	dc_method;
 	isc_result_t	exp_result;
-	size_t		exp_data_len;
 	char		*p;
 	FILE		*fp;
 
@@ -2294,12 +2250,11 @@ t_dns_name_towire_x(const char *testfile, size_t buflen) {
 
 				dc_method = t_dc_method_fromtext(Tokens[3]);
 				exp_result = t_dns_result_fromtext(Tokens[4]);
-				exp_data_len = strtoul(Tokens[3], NULL, 10);
 
 				result = test_dns_name_towire(Tokens[0],
 							      dc_method,
 							      Tokens[2],
-							      exp_data_len,
+							      atoi(Tokens[3]),
 							      exp_result,
 							      buflen);
 			} else {
